MODULE compute_divergence_mod
  USE grid_param
  USE prec, ONLY : rstd
  IMPLICIT NONE
  PRIVATE

#include "../unstructured/unstructured.h90"

  PUBLIC :: divergence, compute_divergence_hex, compute_divergence_unst

CONTAINS

#ifdef BEGIN_DYSL

KERNEL(compute_divergence)

  FORALL_CELLS_EXT()
    ON_PRIMAL
      div_ij = 0.d0
      FORALL_EDGES
         div_ij = div_ij + SIGN*ue(EDGE)*LE
      END_BLOCK
      div(CELL) = div_ij / AI
    END_BLOCK
  END_BLOCK

END_BLOCK

#endif END_DYSL

  SUBROUTINE divergence(f_ue,f_div)
    USE icosa
    USE compute_diagnostics_mod, ONLY : compute_divergence
    TYPE(t_field), POINTER :: f_ue(:)
    TYPE(t_field), POINTER :: f_div(:)
  
    REAL(rstd), POINTER :: ue(:,:)
    REAL(rstd), POINTER :: div(:,:)
    INTEGER :: ind

    CALL transfert_request(f_ue,req_e1_vect)
   
    DO ind=1,ndomain
      IF (.NOT. assigned_domain(ind)) CYCLE
      CALL swap_dimensions(ind)
      CALL swap_geometry(ind)
      ue=f_ue(ind)
      div=f_div(ind)
      CALL compute_divergence(ue, div)
    ENDDO
    
  END SUBROUTINE divergence

!-------------- Wrappers for F2008 conformity -----------------

  SUBROUTINE compute_divergence_unst(ue,div)
    REAL(rstd),INTENT(IN)  :: ue(:,:)
    REAL(rstd),INTENT(OUT) :: div(:,:)
    CALL compute_divergence_unst_(ue,div)
  END SUBROUTINE compute_divergence_unst

  SUBROUTINE compute_divergence_hex(ue,div)
    REAL(rstd),INTENT(IN)  :: ue(:,:)
    REAL(rstd),INTENT(OUT) :: div(:,:)
    CALL compute_divergence_hex_(ue,div)
  END SUBROUTINE compute_divergence_hex

!--------------------------------------------------------------

  SUBROUTINE compute_divergence_unst_(ue,div)
    USE ISO_C_BINDING, only : C_DOUBLE, C_FLOAT
    USE geometry, ONLY : le, Ai
    USE data_unstructured_mod, ONLY : primal_deg, primal_edge, primal_ne
    FIELD_U    :: ue
    FIELD_MASS :: div
    DECLARE_INDICES
    DECLARE_EDGES
    DECLARE_VERTICES
    NUM :: div_ij
#include "../kernels_unst/compute_divergence.k90"
  END SUBROUTINE compute_divergence_unst_

  SUBROUTINE compute_divergence_hex_(ue,div)
    USE icosa
    USE omp_para
    REAL(rstd),INTENT(IN)  :: ue(3*iim*jjm,llm)
    REAL(rstd),INTENT(OUT) :: div(iim*jjm,llm)
    REAL(rstd) :: div_ij
    INTEGER :: ij,l

#include "../kernels_hex/compute_divergence.k90"
  END SUBROUTINE compute_divergence_hex_

END MODULE compute_divergence_mod
